@@ -1,3 +1,5 @@
+from aiotfm.errors import PacketError, PacketTooLarge, XXTEAInvalidPacket, XXTEAInvalidKeys
+
 class Packet:
 	def __init__(self, buffer=None):
 		if buffer is None:
@@ -9,6 +11,7 @@ class Packet:
 		self.pos = 0
 
 		self.exported = False
+		self._fp = 0
 
 	def __repr__(self):
 		return '<Packet {!r}>'.format(bytes(self))
@@ -18,51 +21,32 @@ class Packet:
 
 	@classmethod
 	def new(cls, c, cc=None):
+		"""Create a new instance of Packet initialized by two bytes: c and cc."""
 		msg = cls()
+		if isinstance(c, (tuple, list)):
+			c, cc = c
 		if cc is None:
-			msg.write16(c)
-		else:
-			msg.write8(c).write8(cc)
-
-		return msg
+			return msg.write16(c)
+		return msg.write8(c).write8(cc)
 
 	def copy(self, pos=False):
-		p = Packet(self.buffer[:])
+		"""Returns a copy of the Packet"""
+		p = Packet(self.buffer.copy())
 		if pos:
 			p.pos = self.pos
 		return p
 
-	def unpack(self, fmt):
-		results = []
-		formats = {
-			'B': self.read8,
-			'H': self.read16,
-			'I': self.read24,
-			'L': self.read32,
-			's': self.readString
-		}
-		times = 1
-		for c in fmt:
-			if c.isdigit():
-				if times==1:
-					times = int(c)
-				else:
-					times = times * 10 + int(c)
-			else:
-				for i in range(times):
-					results.append(formats[c]())
-				times = 1
-
-		return results
-
 	def readBytes(self, nbr=1):
+		"""Read raw bytes from the buffer."""
 		self.pos += nbr
 		return self.buffer[self.pos-nbr:self.pos]
 
 	def readCode(self):
+		"""Read two bytes: c and cc."""
 		return self.read8(), self.read8()
 
 	def read8(self):
+		"""Read a single byte from the buffer."""
 		self.pos += 1
 		try:
 			return self.buffer[self.pos-1]
@@ -70,15 +54,19 @@ class Packet:
 			return 0
 
 	def read16(self):
-		return (self.read8() << 8) + self.read8()
+		"""Read a short (two bytes) from the buffer"""
+		return (self.read8() << 8) | self.read8()
 
 	def read24(self):
-		return (self.read16() << 8) + self.read8()
+		"""Read three bytes from the buffer"""
+		return (self.read16() << 8) | self.read8()
 
 	def read32(self):
-		return (self.read24() << 8) + self.read8()
+		"""Read an int (four bytes) from the buffer"""
+		return (self.read24() << 8) | self.read8()
 
 	def readBool(self):
+		"""Read a boolean (one byte) from the buffer"""
 		return self.read8()==1
 
 	def readString(self):
@@ -90,6 +78,7 @@ class Packet:
 		return self.readString().decode()
 
 	def writeBytes(self, bytes):
+		"""Write raw bytes to the buffer"""
 		if isinstance(bytes, Packet):
 			self.buffer.extend(bytes.buffer)
 		else:
@@ -97,31 +86,43 @@ class Packet:
 		return self
 
 	def writeCode(self, c, cc):
+		"""Write two bytes: c and cc."""
 		return self.write8(c).write8(cc)
 
 	def write8(self, value):
+		"""Write a single byte to the buffer"""
 		self.buffer.append(value&0xff)
 		return self
 
 	def write16(self, value):
+		"""Write a short (two bytes) to the buffer"""
 		return self.write8(value>>8).write8(value)
 
 	def write24(self, value):
+		"""Write three bytes to the buffer"""
 		return self.write16(value>>8).write8(value)
 
 	def write32(self, value):
+		"""Write an int (four bytes) to the buffer"""
 		return self.write24(value>>8).write8(value)
 
 	def writeBool(self, value):
+		"""Write a boolean (one byte) to the buffer"""
 		return self.write8(1 if value else 0)
 
 	def writeString(self, string):
+		"""Write a string to the buffer"""
 		if isinstance(string, str):
 			string = string.encode()
 		return self.write16(len(string)).writeBytes(string)
 
+	def writeUTF(self, string):
+		"""Write a string to the buffer. Alias for .writeString"""
+		return self.writeString(string)
+
 	def export(self, fp=0):
-		if self.exported:
+		"""Generates the header then converts the whole packet to bytes and returns it."""
+		if self.exported and self._fp == fp:
 			return self.bytes
 
 		m = Packet()
@@ -133,24 +134,29 @@ class Packet:
 		elif size<=0xffffff:
 			m.write8(3).write24(size)
 		else:
-			raise Exception('Packet too long')
+			raise PacketTooLarge("The Packet maximum size of 16777215 has been exceeded.")
 		m.write8(fp)
 
 		self.bytes = bytes(m.buffer + self.buffer)
 		self.exported = True
+		self._fp = fp
 
 		return self.bytes
 
-	def xor_cipher(self, keys, fp):
+	def xor_cipher(self, key, fp):
+		"""Cipher the packet with the XOR algorithm."""
 		fp += 1
 		ccc = self.readBytes(2)
-		tmp = bytearray([(byte^keys[(fp+i)%20])&0xff for i, byte in enumerate(self.buffer[2:])])
+		tmp = bytearray([(byte^key[(fp+i)%20])&0xff for i, byte in enumerate(self.buffer[2:])])
 		self.buffer = ccc+tmp
 		return self
 
-	def cipher(self, keys):
+	def cipher(self, key):
+		"""Cipher the packet with the XXTEA algorithm."""
 		if len(self.buffer)<2:
-			raise Exception()
+			raise XXTEAInvalidPacket("The Packet is empty.")
+		if len(key)<4:
+			raise XXTEAInvalidKeys(str(key))
 		while len(self.buffer)<10:
 			self.write8(0)
 
@@ -160,7 +166,7 @@ class Packet:
 		for i in range(length//4+(length%4>0)):
 			chunks.append(self.read32())
 
-		chunks = encode_chunks(chunks, len(chunks), keys)
+		chunks = xxtea_encode(chunks, len(chunks), key)
 
 		packet = Packet.new(ccc).write16(len(chunks))
 		for chunk in chunks:
@@ -169,25 +175,17 @@ class Packet:
 		self.buffer = packet.buffer
 		return self
 
-def encode_chunks(v, n, keys):
-	DELTA = 0x9e3779b9
-	def MX():
-		return int(((z>>5)^(y<<2)) + ((y>>3)^(z<<4))^(sum^y) + (keys[(p & 3)^e]^z))
+DELTA = 0x9e3779b9
 
-	y = v[0]
+def xxtea_encode(v, n, key):
+	"""https://en.wikipedia.org/wiki/XXTEA"""
+	cycles = 6 + 52//n
 	sum = 0
-	if n > 1:
-		z = v[n - 1]
-		q = int(6 + 52 / n)
-	while q > 0:
-		q -= 1
+	z = v[-1]
+	for _ in range(cycles):
 		sum = (sum + DELTA) & 0xffffffff
-		e = ((sum >> 2) & 0xffffffff) & 3
-		p = 0
-		while p < n - 1:
-			y = v[p + 1]
-			z = v[p] = (v[p] + MX()) & 0xffffffff
-			p += 1
-		y = v[0]
-		z = v[n - 1] = (v[n - 1] + MX()) & 0xffffffff
+		e = sum >> 2 & 3
+		for p in range(n):
+			y = v[(p+1)%n]
+			z = v[p] = (v[p] + (((z>>5^y<<2) + (y>>3^z<<4)) ^ ((sum^y) + (key[(p&3)^e] ^ z))))&0xffffffff
 	return v
\ No newline at end of file
